﻿#!/usr/bin/perl -w
use strict;
use warnings;
use utf8;
use open qw(:std :utf8);

use Getopt::Long;
use Pod::Usage;


# Filter_3_05_add_basic_xml.pl
# 2010-08-15
# Wolfgang Schmidle

# what does it do?
# input
# output

my $name = "Filter_3_05_add_basic_xml";
my $textfilterParameters = "Filter_parameters/$name.txt";


# get command line options

my $urlbase = ""; 

my $man = 0;
my $help = 0;
my $parameterformat = 'urlbase=s,$'."\n";

sub readParameters {

	GetOptions("urlbase=s" => \$urlbase,
			   "parameterformat" => sub { print $parameterformat; exit; },
			   'help|?' => \$help, man => \$man) or pod2usage(2);
	
	pod2usage(1) if $help;
	pod2usage(-exitstatus => 0, -verbose => 2) if $man;
}

sub isTextfilter {
	return (exists $ENV{'BB_DOC_NAME'})
}

readParameters();

# ohne Dateiname: brich mit USAGE ab. 
unless ($#ARGV > -1) { pod2usage(1); } 


# text input

my @text;
while(<>) { push @text, $_; }


# read the textfilter parameters

if (isTextfilter()) {

	open (PARA, $textfilterParameters) or die "3.05, used as textfilter: can't find the parameter list!\n";
	@ARGV = <PARA>;
	close(PARA);
	chomp @ARGV;

	readParameters();
}


# evaluate and remove the "metadata" block

my %metadata;
my $text = "";

my $inMetadata = 0;

foreach (@text) {
	last if m!<pb[ ^]!;	
	if (m!metadata:!) { 
		$inMetadata = 1;
		$_ = "";
		next;
	}
	if ($inMetadata) {
		if (m!^ *$!) {
			$_ = "";
			last;
		}

		chomp;
		
		if (m!URLbase +(.+)!) { 
			$urlbase = $1; 
			$_ = "";
			next;
		}
	
		if (m!<text[ >]!) { 
			$text = $_;
			$_ = "";
			next;			
		}
	
		if (m!<([^ />]+)!) { 
			if (exists $metadata{$1}) { 
				$metadata{$1} .= "\n    " . $_;
			} else {
				$metadata{$1} = $_;
			}
		}
				
		$_ = "";
	}
}

$metadata{"dcterms:identifier"} = "<dcterms:identifier>ECHO:xxxxxxxx.xml</dcterms:identifier>" unless exists $metadata{"dcterms:identifier"};

$metadata{"dcterms:title"} = '<dcterms:title xml:lang="xx">xxx</dcterms:title>' unless exists $metadata{"dcterms:title"};
$metadata{"dcterms:alternative"} = "" unless exists $metadata{"dcterms:alternative"};
$metadata{"dcterms:creator"} = "<dcterms:creator>xxx, xxx</dcterms:creator>" unless exists $metadata{"dcterms:creator"};
$metadata{"dcterms:contributor"} = "" unless exists $metadata{"dcterms:contributor"};
$metadata{"dcterms:publisher"} = "" unless exists $metadata{"dcterms:publisher"};
$metadata{"dcterms:language"} = '<dcterms:language xsi:type="dcterms:ISO639-3">xxx</dcterms:language>' unless exists $metadata{"dcterms:language"};
$metadata{"dcterms:date"} = '<dcterms:date xsi:type="dcterms:W3CDTF">xxxx</dcterms:date>' unless exists $metadata{"dcterms:date"};
$metadata{"dcterms:description"} = "" unless exists $metadata{"dcterms:description"};

$metadata{"dcterms:rights"} = "<dcterms:rights>open access</dcterms:rights>" unless exists $metadata{"dcterms:rights"};
$metadata{"dcterms:license"} = "<dcterms:license>http://echo.mpiwg-berlin.mpg.de/policy/oa_basics/declaration</dcterms:license>" unless exists $metadata{"dcterms:license"};
$metadata{"dcterms:accessRights"} = "<dcterms:accessRights>free</dcterms:accessRights>" unless exists $metadata{"dcterms:accessRights"};
$metadata{"dcterms:rightsHolder"} = "" unless exists $metadata{"dcterms:rightsHolder"};
$metadata{"dcterms:provenance"} = "" unless exists $metadata{"dcterms:provenance"};
$metadata{"dcterms:dateCopyrighted"} = "" unless exists $metadata{"dcterms.dateCopyrighted"};

$metadata{"font"} = "" unless exists $metadata{"font"};
$metadata{"font-family"} = "" unless exists $metadata{"font-family"};
$metadata{"time-period"} = "" unless exists $metadata{"time-period"};
$metadata{"display-parameters"} = "" unless exists $metadata{"display-parameters"};
$metadata{"echolink"} = "" unless exists $metadata{"echolink"};
$metadata{"echodir"} = "" unless exists $metadata{"echodir"};

$text = '<text xml:lang="xx" type="free">' unless $text ne "";


# evaluate and remove the "parameters" block

my $parameters = "";
my $inParameters = 0;

foreach (@text) {
	last if m!<pb[ ^]!;	
	if (m!parameters:!) { 
		$inParameters  = 1; 
		$_ = "";
		next;
	}
	if ($inParameters) {
		if (m!^ *$!) {
			$_ = "";
			last;
		}
		$parameters .= $_;
		$_ = "";
	}
}

chomp $parameters;
$parameters =~ s!\n! !g;

if ($parameters ne "") { $parameters = "<parameters>$parameters</parameters>"; }


# evaluate and remove the "log" block

my $log = "";
my $inLog = 0;

foreach (@text) {
	last if m!<pb[ ^]!;	
	if (m!log:!) { 
		$inLog = 1; 
		$_ = "";
		next;
	}
	if ($inLog) {
		if (m!^ *$!) {
			$_ = "";
			last;
		}
		$log .= $_;
		$_ = "";
	}
}

chomp $log;
$log =~ s!\n!\n      !g;

$log =~ s!&!&amp;!g;
$log =~ s!<!&lt;!g;
$log =~ s!>!&gt;!g;

if ($log ne "") { $log = "<log>$log</log>"; }


# text output

#print <<PREAMBLE;
# my @preamble = <<PREAMBLE =~ m!(\S.*\S)!g;
my $preamble = <<PREAMBLE;
<?xml version="1.0" encoding="UTF-8"?>

<!DOCTYPE echo [

<!ATTLIST head         xml:space  (preserve)  #FIXED "preserve">
<!ATTLIST s            xml:space  (preserve)  #FIXED "preserve">
<!ATTLIST caption      xml:space  (preserve)  #FIXED "preserve">
<!ATTLIST description  xml:space  (preserve)  #FIXED "preserve">
<!ATTLIST variables    xml:space  (preserve)  #FIXED "preserve">

<!ENTITY  URLbase "$urlbase">
]> 

<echo xmlns="http://www.mpiwg-berlin.mpg.de/ns/echo/1.0/"
  xmlns:de="http://www.mpiwg-berlin.mpg.de/ns/de/1.0/"
  xmlns:dcterms="http://purl.org/dc/terms"
  xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
  xmlns:echo="http://www.mpiwg-berlin.mpg.de/ns/echo/1.0/"
  xmlns:xhtml="http://www.w3.org/1999/xhtml"
  xmlns:mml="http://www.w3.org/1998/Math/MathML"
  xmlns:xlink="http://www.w3.org/1999/xlink"
  version="1.0">
  <metadata>
    $metadata{"dcterms:identifier"}
    
    $metadata{"dcterms:creator"}
    $metadata{"dcterms:contributor"}
    $metadata{"dcterms:title"}
    $metadata{"dcterms:alternative"}
    $metadata{"dcterms:date"}
    $metadata{"dcterms:language"}
    $metadata{"dcterms:publisher"}
    $metadata{"dcterms:description"}
    
    $metadata{"dcterms:rights"}
    $metadata{"dcterms:license"}
    $metadata{"dcterms:accessRights"}
    $metadata{"dcterms:rightsHolder"}
    $metadata{"dcterms:provenance"}
    $metadata{"dcterms:dateCopyrighted"}
    
    $metadata{"font"}
    $metadata{"font-family"}
    $metadata{"time-period"}
    $metadata{"display-parameters"}
    $metadata{"echolink"}
    $metadata{"echodir"}
    
    $parameters
    $log
  </metadata>
  $text
    <div type="body" level="0" n="0">
PREAMBLE

$preamble =~ s! +\n!!g; # lösche die Zier-Leerzeilen, behalte die echten Leerzeilen
print $preamble;

print @text;

print "    </div>\n  </text>\n</echo>\n";


# TO DO:
# das spezifische Wissen über einzelne Metadaten-Einträge ist irgendwo im Skript versteckt. 
# --> Hilfsdatei verwenden? Oder einfach umstrukturieren?

# weitere links, etc.

__END__

=head1 NAME

Filter_2_03_find_forbidden_characters

=head1 SYNOPSIS

XXX noch nicht angepasst

perl Filter_2_03_find_forbidden_characters.pl [options] [file]

 Options:
   -help       brief help message
   -man        full documentation
   -dir        path to the additional files 
   -forbidden   adds a list of forbidden characters

=head1 OPTIONS
=over 8

=item B<-help>
Print a brief help message and exits.

=item B<-man>
Prints the manual page and exits.

=item B<-forbidden>
Adds a list of forbidden characters. The order of lists is important. 

=back

=head1 DESCRIPTION

B<Filter_2_03_find_forbidden_characters> will check the input file for forbidden characters.

For the parameter fromat in the helper files see ...

=head1 TO DO

...

=cut
