﻿#!/usr/bin/perl -w
use strict;
use warnings;
use utf8;
use open qw(:std :utf8);
use integer;


# Filter_4_06_tables.pl
# 2010-08-17
# Wolfgang Schmidle

# This Script transforms <tb> into xhtml


# Version für Chinesisch
# akzeptiert <p> als Markierung einer Table-Zeile

# text input

my @text;
while(<>) { push @text, $_; }


# go through the text

my $inTable = 0;
my $i = 0;

# Annahmen des Skripts:
# <toc> wurde in 3.06 zu <div type="toc" level="0" n="0">
# <ind> wurde in 3.06 zu <div type="index" level="0" n="0">
# <tb> ist immer noch <tb>
# (<h> wurde in 3.06 zu <head>)
# Unterschied: <tb> ist einfach eine Tabelle, <toc> und <ind> sind semantische Beschreibungen,
# die in den meisten Fällen aus Tabellen bestehen 
# das Skript erklärt alle Textzeilen, die weder in <head> noch in <p> sind, zu Tabellenzeilen
# Das Skript ist nicht in der Lage, Footer zu erkennen.

# Beachte, dass Formax Tabellenzeilen manchmal auch mit <p> markiert! 
# Diese werden kommentarlos entfernt, was zu falschen Ergebnissen führen kann


sub zeileUmwandeln {

	# <head> bewirkt neue Tabelle
	s!(<head[ >])!</xhtml:table>\n$1!;
	s!</head>!</head>\n<xhtml:table>!;
	return if m!</?xhtml:table>!;

	# normale Tabellenzeile
	$_ = "<xhtml:tr>\n<xhtml:td>" . $_ . "</xhtml:td></xhtml:tr>\n";
	s! ?# ?!</xhtml:td><xhtml:td>!g;
	s!\\\\!<lb/>!g;
	
	# korrekte Formatierung
	s!<xhtml:tr>!  <xhtml:tr>!g;
	s!</xhtml:tr>!  </xhtml:tr>!g;
	s!<xhtml:td>!    <xhtml:td>!g;
	s!</xhtml:td>!</xhtml:td>\n!g;
	s!<xhtml:td>!<xhtml:td xml:space="preserve">!g;
}


# <tb>

foreach (@text) {
	
	$i++;

	if (m!<tb>!) {
		$_ = "";
		$inTable = 1;
	}
	if ($inTable == 1) {
		next if $text[$i] =~ m!<head[ >]!;
		$_ .= "<xhtml:table>\n";
		$inTable = 2;		
		next;
	}
	next unless $inTable;

	if (s!</tb>!</xhtml:table>!) { 
		$inTable = 0;
		next;
	}

	chomp;

	s!^<p>(.+?)</p>$!$1!; # entferne <p>
	zeileUmwandeln($_);
}


# ehemaliges <toc>, <ind>

$i = 0;

foreach (@text) {
	
	$i++;
	
	if (m!<div type="toc" level="0" n="0">!) { 
		$inTable = 1;
	}
	if ($inTable == 1) {
		next if $text[$i] =~ m!<head[ >]!;
		$_ .= "<xhtml:table>\n";
		$inTable = 2;		
		next;
	}
	next unless $inTable;
	
	if (m!</div>!) { 
		$_ = "</xhtml:table>\n" . $_;
		$inTable = 0;
		next;
	}


	chomp;
	
	# hier wird jetzt noch nicht einfach <p> entfernt. Konkrete Fälle abwarten?
	
	zeileUmwandeln();
}


# text output

print @text;


# TO DO:
# ...

